import type { MessageExpressionPart } from '../formatted-parts.js';
import type { MessageFunctionContext, MessageValue } from './index.js';
/** @beta */
export interface MessageDateTime extends MessageValue {
    readonly type: 'datetime';
    readonly source: string;
    readonly locale: string;
    readonly options: Readonly<Intl.DateTimeFormatOptions>;
    toParts(): [MessageDateTimePart];
    toString(): string;
    valueOf(): Date;
}
/** @beta */
export interface MessageDateTimePart extends MessageExpressionPart {
    type: 'datetime';
    source: string;
    locale: string;
    parts: Intl.DateTimeFormatPart[];
}
/**
 * `datetime` accepts an optional Date, number or string as its input
 * and formats it with the same options as
 * {@link https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Intl/DateTimeFormat | Intl.DateTimeFormat}.
 * If not given any argument, the current date/time is used.
 *
 * @beta
 */
export declare function datetime({ localeMatcher, locales, source }: MessageFunctionContext, options: Record<string, unknown>, input?: unknown): MessageDateTime;
